/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/applesingle.h                                        *
 * Created:     2006-08-18 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2006-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef APPLESINGLE_H
#define APPLESINGLE_H 1


#include <lib/macfile.h>


#define AS_ENTRY_DATA        1
#define AS_ENTRY_RSRC        2
#define AS_ENTRY_NAME        3
#define AS_ENTRY_COMMENT     4
#define AS_ENTRY_ICON        5
#define AS_ENTRY_CICON       6
#define AS_ENTRY_TIME        8
#define AS_ENTRY_FINDER_INFO 9
#define AS_ENTRY_FILE_INFO   10


typedef struct {
	unsigned long id;
	unsigned long ofs;
	unsigned long size;
} as_entry_t;

typedef struct {
	FILE          *fp;

	unsigned long magic;
	unsigned long version;

	unsigned      entcnt;
	as_entry_t    *entry;
} applesingle_t;


void as_init (applesingle_t *as, FILE *fp);
void as_free (applesingle_t *as);

applesingle_t *as_new (FILE *fp);
void as_del (applesingle_t *as);

int as_add_entry (applesingle_t *as, unsigned long id, unsigned long size, unsigned long ofs);
void as_fix_offsets (applesingle_t *as);

unsigned as_get_entry_count (applesingle_t *as);
unsigned long as_get_index_id (applesingle_t *as, unsigned idx);
unsigned long as_get_index_size (applesingle_t *as, unsigned idx);
unsigned long as_get_index_offset (applesingle_t *as, unsigned idx);

unsigned long as_get_entry_size (applesingle_t *as, unsigned long id);
unsigned long as_get_entry_offset (applesingle_t *as, unsigned long id);

unsigned long as_get_magic (applesingle_t *as);
unsigned long as_get_version (applesingle_t *as);

int as_is_applesingle (const applesingle_t *as);
int as_is_appledouble (const applesingle_t *as);

int as_read_header (applesingle_t *as);
int as_write_header (applesingle_t *as, int ad);

int as_read (applesingle_t *as, void *buf, unsigned long id, unsigned long ofs, unsigned long cnt);
int as_write (applesingle_t *as, const void *buf, unsigned long id, unsigned long ofs, unsigned long cnt);

unsigned long as_time_to_mac (unsigned long val);
unsigned long as_time_from_mac (unsigned long val);

void as_get_macfile (applesingle_t *as, macfile_t *mf);


#endif
