/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/buffer.c                                             *
 * Created:     2005-02-17 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2005-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <lib/buffer.h>


unsigned char get_uint8_be (const void *buf, unsigned i)
{
	const unsigned char *tmp;

	tmp = (const unsigned char *) buf + i;

	return (tmp[0] & 0xff);
}

unsigned short get_uint16_be (const void *buf, unsigned i)
{
	unsigned short      r;
	const unsigned char *tmp;

	tmp = (const unsigned char *) buf + i;

	r = tmp[0] & 0xff;
	r = (r << 8) | (tmp[1] & 0xff);

	return (r);
}

unsigned long get_uint32_be (const void *buf, unsigned i)
{
	unsigned long       r;
	const unsigned char *tmp;

	tmp = (const unsigned char *) buf + i;

	r = tmp[0] & 0xff;
	r = (r << 8) | (tmp[1] & 0xff);
	r = (r << 8) | (tmp[2] & 0xff);
	r = (r << 8) | (tmp[3] & 0xff);

	return (r);
}

void set_uint8_be (void *buf, unsigned i, unsigned char v)
{
	unsigned char *tmp;

	tmp = (unsigned char *) buf + i;

	tmp[0] = v & 0xff;
}

void set_uint16_be (void *buf, unsigned i, unsigned short v)
{
	unsigned char *tmp;

	tmp = (unsigned char *) buf + i;

	tmp[0] = (v >> 8) & 0xff;
	tmp[1] = v & 0xff;
}

void set_uint32_be (void *buf, unsigned i, unsigned long v)
{
	unsigned char *tmp;

	tmp = (unsigned char *) buf + i;

	tmp[0] = (v >> 24) & 0xff;
	tmp[1] = (v >> 16) & 0xff;
	tmp[2] = (v >> 8) & 0xff;
	tmp[3] = v & 0xff;
}
