/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/dc42.c                                               *
 * Created:     2009-10-12 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2009-2011 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <lib/buffer.h>
#include <lib/dc42.h>
#include <lib/strings.h>


void dc42_init (void *hdr)
{
	memset (hdr, 0, 84);

	set_uint16_be (hdr, 82, 0x0100);
}

int dc42_read (void *hdr, FILE *fp)
{
	if (fread (hdr, 1, 84, fp) != 84) {
		return (1);
	}

	return (0);
}

int dc42_write (const void *hdr, FILE *fp)
{
	if (fwrite (hdr, 1, 84, fp) != 84) {
		return (1);
	}

	return (0);
}

const unsigned char *dc42_get_name_pstr (const void *hdr)
{
	return (hdr);
}

int dc42_set_name_pstr (void *hdr, const void *str)
{
	const unsigned char *s;

	s = str;

	if (s[0] > 63) {
		return (1);
	}

	memset (hdr, 0, 64);
	memcpy (hdr, s, s[0] + 1);

	return (0);
}

char *dc42_get_name_cstr (const void *hdr)
{
	return (mac_pstring_decode (hdr, "/"));
}

int dc42_set_name_cstr (void *hdr, const char *str)
{
	memset (hdr, 0, 64);

	return (mac_pstring_encode_buf (hdr, 64, str));
}


unsigned long dc42_get_data_size (const void *hdr)
{
	return (get_uint32_be (hdr, 64));
}

void dc42_set_data_size (void *hdr, unsigned long v)
{
	set_uint32_be (hdr, 64, v);

	/*
	 * Don't know what these mean, but these are the values that
	 * DiskCopy 6.3 uses (and requires).
	 */
	if (v == (400UL * 1024UL)) {
		set_uint16_be (hdr, 80, 0x0002);
	}
	else if (v == (800UL * 1024UL)) {
		set_uint16_be (hdr, 80, 0x0122);
	}
	else if (v == (720UL * 1024UL)) {
		set_uint16_be (hdr, 80, 0x0222);
	}
	else if (v == (1440UL * 1024UL)) {
		set_uint16_be (hdr, 80, 0x0322);
	}
	else {
		set_uint16_be (hdr, 80, 0);
	}
}

unsigned long dc42_get_tag_size (const void *hdr)
{
	return (get_uint32_be (hdr, 68));
}

void dc42_set_tag_size (void *hdr, unsigned long v)
{
	set_uint32_be (hdr, 68, v);
}

unsigned long dc42_get_data_checksum (const void *hdr)
{
	return (get_uint32_be (hdr, 72));
}

void dc42_set_data_checksum (void *hdr, unsigned long v)
{
	set_uint32_be (hdr, 72, v);
}

unsigned long dc42_get_tag_checksum (const void *hdr)
{
	return (get_uint32_be (hdr, 76));
}

void dc42_set_tag_checksum (void *hdr, unsigned long v)
{
	set_uint32_be (hdr, 76, v);
}

unsigned long dc42_calc_checksum (const void *buf, unsigned long cnt, unsigned long chk)
{
	unsigned            val;
	const unsigned char *tmp = buf;

	cnt >>= 1;

	while (cnt > 0) {
		val = tmp[0] & 0xff;
		val = (val << 8) | (tmp[1] & 0xff);
		chk = (chk + val) & 0xffffffff;
		chk = ((chk >> 1) | (chk << 31)) & 0xffffffff;
		cnt -= 1;
		tmp += 2;
	}

	return (chk);
}
