/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/file.c                                               *
 * Created:     2007-12-02 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2007-2012 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "file.h"


FILE *mac_open_out (const char *name, const char *mode, int std)
{
	if (name == NULL) {
		if (std) {
			return (stdout);
		}

		return (NULL);
	}

	if (strcmp (name, "-") == 0) {
		return (stdout);
	}

	return (fopen (name, mode));
}

FILE *mac_open_inp (const char *name, const char *mode, int std)
{
	if (name == NULL) {
		if (std) {
			return (stdin);
		}

		return (NULL);
	}

	if (strcmp (name, "-") == 0) {
		return (stdin);
	}

	return (fopen (name, mode));
}

void mac_close (FILE *fp)
{
	if (fp == NULL) {
		return;
	}

	if (fp == stdin) {
		return;
	}

	if ((fp == stdout) || (fp == stderr)) {
		return;
	}

	fclose (fp);
}

int mac_set_pos (FILE *fp, unsigned long ofs)
{
	if (fseek (fp, ofs, SEEK_SET)) {
		return (1);
	}

	return (0);
}

int mac_get_pos (FILE *fp, unsigned long *ofs)
{
	long tmp;

	tmp = ftell (fp);

	if (tmp == -1) {
		return (1);
	}

	*ofs = tmp;

	return (0);
}

int mac_read (FILE *fp, void *buf, unsigned long ofs, unsigned long cnt)
{
	if (fseek (fp, ofs, SEEK_SET)) {
		return (1);
	}

	if (fread (buf, cnt, 1, fp) != 1) {
		return (1);
	}

	return (0);
}

int mac_write (FILE *fp, const void *buf, unsigned long ofs, unsigned long cnt)
{
	if (fseek (fp, ofs, SEEK_SET)) {
		return (1);
	}

	if (fwrite (buf, cnt, 1, fp) != 1) {
		return (1);
	}

	return (0);
}
