/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/fourcc.c                                             *
 * Created:     2004-03-04 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2004-2011 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "fourcc.h"


static
int is_hex_digit (char c)
{
	if ((c >= '0') && (c <= '9')) {
		return (1);
	}

	if ((c >= 'a') && (c <= 'f')) {
		return (1);
	}

	if ((c >= 'A') && (c <= 'F')) {
		return (1);
	}

	return (0);
}

static
unsigned get_hex_digit (char c)
{
	if ((c >= '0') && (c <= '9')) {
		return (c - '0');
	}

	if ((c >= 'a') && (c <= 'f')) {
		return (c - 'a' + 10);
	}

	if ((c >= 'A') && (c <= 'F')) {
		return (c - 'A' + 10);
	}

	return (0);
}

int str_set_fourcc (char *str, unsigned max, unsigned long fcc)
{
	unsigned i, j, c;

	j = 0;
	for (i = 0; i < 4; i++) {
		c = (fcc >> 24) & 0xff;

		if ((c <= 0x20) || (c > 0x7e) || (c == '=') || (c == '"')) {
			unsigned c1, c2;

			if ((j + 3) >= max) {
				return (1);
			}

			c1 = (c >> 4) & 0x0f;
			c2 = c & 0x0f;

			str[j] = '=';
			str[j + 1] = (c1 <= 9) ? ('0' + c1) : ('a' + c1 - 10);
			str[j + 2] = (c2 <= 9) ? ('0' + c2) : ('a' + c2 - 10);

			j += 3;
		}
		else {
			if ((j + 1) >= max) {
				return (1);
			}

			str[j++] = c;
		}

		fcc <<= 8;
	}

	str[j] = 0;

	return (0);
}

int str_get_fourcc (const char *str, unsigned long *fcc)
{
	unsigned i;

	*fcc = 0;

	for (i = 0; i < 4; i++) {
		if (str[0] == 0) {
			*fcc = *fcc << 8;
		}
		else if (str[0] == '=') {
			if (is_hex_digit (str[1]) && is_hex_digit (str[2])) {
				*fcc = (*fcc << 4) | get_hex_digit (str[1]);
				*fcc = (*fcc << 4) | get_hex_digit (str[2]);
				str += 3;
			}
			else {
				*fcc = (*fcc << 8) | '=';
				str += 1;
			}
		}
		else {
			*fcc = (*fcc << 8) | ((unsigned) str[0] & 0xff);
			str += 1;
		}
	}

	return (0);
}
