/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/hfs.h                                                *
 * Created:     2011-07-05 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2011-2018 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef MAC_LIB_HFS_H
#define MAC_LIB_HFS_H 1


#include <stdio.h>


#define HFS_BOOT_ID   0x4c4b
#define HFS_MDB_ID    0x4244
#define HFS_DEVICE_ID 0x4552
#define HFS_PMAP_ID   0x504d


typedef struct {
	FILE          *fp;

	unsigned long offset;
	unsigned long blocks;

	char          readonly;

	unsigned      ablk_first;
	unsigned      ablk_cnt;
	unsigned long ablk_size;

	unsigned      bitmap_start;
	unsigned      bitmap_size;

	unsigned char boot[1024];
	unsigned char mdb[512];
	unsigned char bitmap[8192];
	unsigned char alt_mdb[512];
} hfs_vol_t;


int hfs_open (hfs_vol_t *hfs, FILE *fp, unsigned long offset, unsigned long blocks);
int hfs_open_pm (hfs_vol_t *hfs, FILE *fp, unsigned long offset, unsigned part);
int hfs_open_dc42 (hfs_vol_t *hfs, FILE *fp);
int hfs_open_auto (hfs_vol_t *hfs, FILE *fp, unsigned long offset, unsigned long blocks, unsigned part);
int hfs_close (hfs_vol_t *hfs, int write);

int hfs_read_block (hfs_vol_t *hfs, void *buf, unsigned long idx, unsigned long cnt);
int hfs_write_block (hfs_vol_t *hfs, const void *buf, unsigned long idx, unsigned long cnt);

unsigned hfs_get_mdb_id (const void *mdb);
void hfs_set_mdb_id (void *mdb, unsigned val);

char *hfs_get_vol_name (const void *mdb);
int hfs_set_vol_name (void *mdb, const char *val);

unsigned long hfs_get_ctime (const void *mdb);
void hfs_set_ctime (void *mdb, unsigned long val);

unsigned long hfs_get_mtime (const void *mdb);
void hfs_set_mtime (void *mdb, unsigned long val);

unsigned long hfs_get_btime (const void *mdb);
void hfs_set_btime (void *mdb, unsigned long val);

unsigned hfs_get_volume_attributes (const void *mdb);
void hfs_set_volume_attributes (void *mdb, unsigned val);

unsigned hfs_get_ablk_count (const void *mdb);
void hfs_set_ablk_count (void *mdb, unsigned val);

unsigned long hfs_get_ablk_size (const void *mdb);
void hfs_set_ablk_size (void *mdb, unsigned long val);

unsigned hfs_get_first_ablk (const void *mdb);
void hfs_set_first_ablk (void *mdb, unsigned val);

unsigned hfs_get_free_ablk (const void *mdb);
void hfs_set_free_ablk (void *mdb, unsigned val);

unsigned long hfs_get_write_count (const void *mdb);
void hfs_set_write_count (void *mdb, unsigned long val);

void hfs_get_finder_info (const void *mdb, void *buf);
void hfs_set_finder_info (void *mdb, const void *buf);

void hfs_update_alt_mdb (hfs_vol_t *hfs);
void hfs_restore_alt_mdb (hfs_vol_t *hfs);


#endif
