/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/hqx.h                                                *
 * Created:     2005-04-07 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2005-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef MAC_LIB_HQX_H
#define MAC_LIB_HQX_H 1


#include <stdio.h>

#include <lib/macfile.h>


typedef struct {
	FILE           *fp;

	int            eof;
	int            rle;

	unsigned       column;

	unsigned       bufidx;
	unsigned       bufcnt;
	unsigned char  buf[3];

	unsigned       runcnt;
	unsigned       runchr;

	unsigned char  fname[256];

	unsigned char  header[32];
} hqx_t;


unsigned char hqx_get_uint8 (const void *buf, unsigned i);
unsigned short hqx_get_uint16 (const void *buf, unsigned i);
unsigned long hqx_get_uint32 (const void *buf, unsigned i);

void hqx_set_uint8 (void *buf, unsigned i, unsigned char v);
void hqx_set_uint16 (void *buf, unsigned i, unsigned short v);
void hqx_set_uint32 (void *buf, unsigned i, unsigned long v);

int hqx_init_read (hqx_t *hqx, FILE *fp);
int hqx_get_byte_raw (hqx_t *hqx, unsigned char *val);
int hqx_get_byte (hqx_t *hqx, unsigned char *val);
int hqx_read (hqx_t *hqx, void *buf, unsigned cnt);

void hqx_init_write (hqx_t *hqx, FILE *fp, int rle);
void hqx_set_byte_raw (hqx_t *hqx, unsigned char c);
void hqx_set_byte (hqx_t *hqx, unsigned char c);
void hqx_write (hqx_t *hqx, const void *buf, unsigned cnt);
void hqx_flush (hqx_t *hqx);

int hqx_read_header (hqx_t *hqx);
int hqx_write_header (hqx_t *hqx);

unsigned short hqx_calc_crc (const hqx_t *hqx);
int hqx_check_crc (const hqx_t *hqx);

const unsigned char *hqx_get_name_pstr (const hqx_t *hqx);
void hqx_set_name_pstr (hqx_t *hqx, const void *str);

char *hqx_get_name_cstr (const hqx_t *hqx);
int hqx_set_name_cstr (hqx_t *hqx, const char *str);

unsigned hqx_get_version (const hqx_t *hqx);
void hqx_set_version (hqx_t *hqx, unsigned val);

unsigned long hqx_get_type (const hqx_t *hqx);
void hqx_set_type (hqx_t *hqx, unsigned long val);

unsigned long hqx_get_creator (const hqx_t *hqx);
void hqx_set_creator (hqx_t *hqx, unsigned long val);

unsigned short hqx_get_finder_flags (const hqx_t *hqx);
void hqx_set_finder_flags (hqx_t *hqx, unsigned short val);

unsigned long hqx_get_data_size (const hqx_t *hqx);
void hqx_set_data_size (hqx_t *hqx, unsigned long val);

unsigned long hqx_get_rsrc_size (const hqx_t *hqx);
void hqx_set_rsrc_size (hqx_t *hqx, unsigned long val);

unsigned short hqx_get_crc (const hqx_t *hqx);
void hqx_set_crc (hqx_t *hqx, unsigned short val);

void hqx_get_macfile (const hqx_t *hqx, macfile_t *mf);
void hqx_set_macfile (hqx_t *hqx, const macfile_t *mf);


#endif
