/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/rsrc.h                                               *
 * Created:     2004-03-21 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2004-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef MAC_LIB_RSRC_H
#define MAC_LIB_RSRC_H 1


#include <stdio.h>


typedef struct rsrc_s {
	unsigned long  type;
	unsigned short id;
	unsigned long  size;
	unsigned char  attr;

	unsigned char  *name;
	unsigned       name_cnt;

	unsigned char  *data;

	FILE           *fp;
	unsigned long  offset;
} rsrc_t;


typedef struct rsrc_type_s {
	unsigned long type;

	unsigned      dupcnt;

	unsigned      rsrc_cnt;
	rsrc_t        **rsrc;
} rsrc_type_t;


typedef struct {
	unsigned      attr;

	unsigned long dupcnt;

	unsigned      type_cnt;
	rsrc_type_t   **type;
} rsrc_file_t;


rsrc_t *rsc_new (unsigned long type, unsigned id);
void rsc_del (rsrc_t *r);
unsigned long rsc_get_type (rsrc_t *r);
void rsc_set_type (rsrc_t *r, unsigned long type);
unsigned rsc_get_id (rsrc_t *r);
void rsc_set_id (rsrc_t *r, unsigned id);
unsigned rsc_get_name_cnt (rsrc_t *r);
unsigned char *rsc_get_name (rsrc_t *r);
int rsc_set_name (rsrc_t *r, const unsigned char *str, unsigned cnt);
unsigned long rsc_get_size (rsrc_t *r);
unsigned char *rsc_get_data (rsrc_t *r);
void rsc_set_data (rsrc_t *r, void *buf, unsigned long cnt);

rsrc_type_t *rt_new (unsigned long type);
void rt_del (rsrc_type_t *rt);
rsrc_t *rt_get_resource (rsrc_type_t *rt, unsigned id);
rsrc_t *rt_add_resource (rsrc_type_t *rt, rsrc_t *rsc);
rsrc_t *rt_new_resource (rsrc_type_t *rt, unsigned id);

/*!***************************************************************************
 * @short Sort the resources in a resource type
 * @param rt     The resource type
 * @param revrsc Sort in reverse order if true
 *****************************************************************************/
void rt_sort (rsrc_type_t *rt, int revrsc);

/*!***************************************************************************
 * @short Remove duplicate resources
 * @param rt     The resource type
 *****************************************************************************/
unsigned rt_remove_duplicates (rsrc_type_t *rt);


rsrc_file_t *rf_new (void);
void rf_del (rsrc_file_t *rf);
rsrc_type_t *rf_get_type (rsrc_file_t *rf, unsigned long type);
rsrc_type_t *rf_add_type (rsrc_file_t *rf, rsrc_type_t *rt);
rsrc_type_t *rf_new_type (rsrc_file_t *rf, unsigned long type);
rsrc_t *rf_add_resource (rsrc_file_t *rf, rsrc_t *r);

/*!***************************************************************************
 * @short Sort the resources and types in a resource file
 * @param rf     The resource file
 * @param revtyp Sort types in reverse order if true
 * @param revrsc Sort resources in reverse order if true
 *****************************************************************************/
void rf_sort (rsrc_file_t *rf, int revtyp, int revrsc);

/*!***************************************************************************
 * @short Remove duplicate resources
 * @param rf     The resource file
 *****************************************************************************/
unsigned long rf_remove_duplicates (rsrc_file_t *rf);

unsigned long rf_get_resource_count (rsrc_file_t *rf);

rsrc_file_t *rf_read (FILE *fp);

int rf_write (rsrc_file_t *rf, FILE *fp);


#endif
