/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/lib/tar.c                                                *
 * Created:     2007-12-01 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2007-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tar.h"


void tar_init (unsigned char *buf)
{
	memset (buf, 0, 512);

	buf[257] = 'u';
	buf[258] = 's';
	buf[259] = 't';
	buf[260] = 'a';
	buf[261] = 'r';
	buf[262] = 0;

	buf[263] = '0';
	buf[264] = '0';
}

static
void tar_set_octal (unsigned char *buf, unsigned idx, unsigned cnt, unsigned long val)
{
	unsigned i;

	idx = idx + cnt - 2;

	buf[idx + 1] = 0;

	for (i = 1; i < cnt; i++) {
		buf[idx] = '0' + (val % 8);
		idx -= 1;
		val = val / 8;
	}
}

int tar_set_fname (unsigned char *buf, const char *fname)
{
	unsigned i, j, n;

	memset (buf, 0, 100);
	memset (buf + 345, 0, 155);

	while (*fname == '/') {
		fname += 1;
	}

	n = strlen (fname);

	while ((n > 0) && (fname[n - 1] == '/')) {
		n -= 1;
	}

	if (n <= 100) {
		memcpy (buf, fname, n);
		return (0);
	}

	i = 1;
	while (i < n) {
		if (fname[i] == '/') {
			if ((i <= 155) && ((n - i - 1) <= 100)) {
				break;
			}
		}

		i += 1;
	}

	if (i >= n) {
		return (1);
	}

	j = i;
	while ((j > 0) && (fname[j - 1] == '/')) {
		j -= 1;
	}

	while (fname[i] == '/') {
		i += 1;
	}

	memcpy (buf, fname + i, n - i);
	memcpy (buf + 345, fname, j);

	return (0);
}

void tar_set_mode (unsigned char *buf, unsigned long mode)
{
	tar_set_octal (buf, 100, 8, mode);
}

int tar_set_user (unsigned char *buf, unsigned long uid, const char *uname)
{
	unsigned n;

	tar_set_octal (buf, 108, 8, uid);

	n = strlen (uname);

	if (n > 31) {
		return (1);
	}

	if (n > 0) {
		memcpy (buf + 265, uname, n);
	}

	return (0);
}

int tar_set_group (unsigned char *buf, unsigned long gid, const char *gname)
{
	unsigned n;

	tar_set_octal (buf, 116, 8, gid);

	n = strlen (gname);

	if (n > 31) {
		return (1);
	}

	if (n > 0) {
		memcpy (buf + 297, gname, n);
	}

	return (0);
}

int tar_set_size (unsigned char *buf, unsigned long val)
{
	tar_set_octal (buf, 124, 12, val);

	return (0);
}

void tar_set_mtime (unsigned char *buf, unsigned long val)
{
	tar_set_octal (buf, 136, 12, val);
}

void tar_set_type (unsigned char *buf, char type)
{
	buf[156] = (unsigned char) type;
}

void tar_set_device (unsigned char *buf, unsigned long maj, unsigned long min)
{
	tar_set_octal (buf, 329, 8, maj);
	tar_set_octal (buf, 337, 8, min);
}

void tar_set_checksum (unsigned char *buf)
{
	unsigned      i;
	unsigned long val;

	for (i = 0; i < 8; i++) {
		buf[148 + i] = 32;
	}

	val = 0;

	for (i = 0; i < 512; i++) {
		val += buf[i];
	}

	tar_set_octal (buf, 148, 8, val);
}
