/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/macc/macc.h                                              *
 * Created:     2011-06-30 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2011-2013 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#ifndef MACC_MAIN_H
#define MACC_MAIN_H 1


#include "config.h"

#include <stdio.h>

#include <lib/macfile.h>


#define MACC_NONE 0
#define MACC_AS   1
#define MACC_HCX  2
#define MACC_HEX  3
#define MACC_HQX  4
#define MACC_MAR  5
#define MACC_MB   6


typedef struct macc_inp_s {
	void *ext;

	void (*close) (struct macc_inp_s *minp);

	int  (*read_header) (struct macc_inp_s *minp, macfile_t *mf);

	int  (*read_data_start) (struct macc_inp_s *minp);
	int  (*read_data) (struct macc_inp_s *minp, void *buf, unsigned long cnt);
	int  (*read_data_done) (struct macc_inp_s *minp);

	int  (*read_rsrc_start) (struct macc_inp_s *minp);
	int  (*read_rsrc) (struct macc_inp_s *minp, void *buf, unsigned long cnt);
	int  (*read_rsrc_done) (struct macc_inp_s *minp);

	int  (*read_comm_start) (struct macc_inp_s *minp);
	int  (*read_comm) (struct macc_inp_s *minp, void *buf, unsigned long cnt);
	int  (*read_comm_done) (struct macc_inp_s *minp);

	FILE *fp;
} macc_inp_t;


typedef struct macc_out_s {
	void *ext;

	void (*close) (struct macc_out_s *minp);

	int  (*write_header) (struct macc_out_s *mout, macfile_t *mf);

	int  (*write_data_start) (struct macc_out_s *mout);
	int  (*write_data) (struct macc_out_s *mout, const void *buf, unsigned long cnt);
	int  (*write_data_done) (struct macc_out_s *mout);

	int  (*write_rsrc_start) (struct macc_out_s *mout);
	int  (*write_rsrc) (struct macc_out_s *mout, const void *buf, unsigned long cnt);
	int  (*write_rsrc_done) (struct macc_out_s *mout);

	int  (*write_comm_start) (struct macc_out_s *mout);
	int  (*write_comm) (struct macc_out_s *mout, const void *buf, unsigned long cnt);
	int  (*write_comm_done) (struct macc_out_s *mout);

	FILE *fp;
} macc_out_t;


extern const char *arg0;


macc_inp_t *macc_inp_as_open (const char *fname, const char *dname);
macc_inp_t *macc_inp_hcx_open (const char *fname);
macc_inp_t *macc_inp_hqx_open (const char *fname);
macc_inp_t *macc_inp_mar_open (const char *fname);
macc_inp_t *macc_inp_mb_open (const char *fname);

macc_out_t *macc_out_as_open (const char *fname, const char *dname);
macc_out_t *macc_out_hcx_open (const char *fname, int compressed);
macc_out_t *macc_out_hqx_open (const char *fname, int rle);
macc_out_t *macc_out_mar_open (const char *fname);
macc_out_t *macc_out_mb_open (const char *fname);


#endif
