/*****************************************************************************
 * mac-utils                                                                 *
 *****************************************************************************/

/*****************************************************************************
 * File name:   src/macrsc/parse.c                                           *
 * Created:     2005-02-17 by Hampa Hug <hampa@hampa.ch>                     *
 * Copyright:   (C) 2005-2009 Hampa Hug <hampa@hampa.ch>                     *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/


#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "parse.h"


int match_space (const char **str)
{
	const char *tmp;

	tmp = *str;

	while ((*tmp == ' ') || (*tmp == '\t')) {
		tmp += 1;
	}

	*str = tmp;

	return (1);
}

int match_eol (const char **str)
{
	const char *tmp;

	match_space (str);

	tmp = *str;

	if ((*tmp == 0) || (*tmp == '#')) {
		return (1);
	}

	return (0);
}

int match_char (const char **str, char c)
{
	if (**str == c) {
		*str += 1;
		return (1);
	}

	return (0);
}

int match_text (const char **str, const char *val)
{
	const char *tmp;

	match_space (str);

	tmp = *str;

	while (*val != 0) {
		if (*tmp != *val) {
			return (0);
		}

		tmp += 1;
		val += 1;
	}

	*str = tmp;

	return (1);
}

int match_string (const char **str, char *val, unsigned max)
{
	const char *tmp;

	match_space (str);

	tmp = *str;

	if (*tmp != '\"') {
		return (0);
	}

	tmp += 1;

	while (max > 0) {
		if (*tmp == '\"') {
			*val = 0;
			*str = tmp + 1;
			return (1);
		}

		if (*tmp == 0) {
			return (0);
		}

		*(val++) = *(tmp++);

		max -= 1;
	}

	return (0);
}

int match_uhex8 (const char **str, unsigned char *val)
{
	unsigned   cnt;
	const char *tmp;

	match_space (str);

	tmp = *str;
	*val = 0;

	cnt = 0;
	while (cnt < 2) {
		if ((*tmp >= '0') && (*tmp <= '9')) {
			*val = (*val << 4) | (*tmp - '0');
		}
		else if ((*tmp >= 'a') && (*tmp <= 'f')) {
			*val = (*val << 4) | (*tmp - 'a' + 10);
		}
		else if ((*tmp >= 'A') && (*tmp <= 'F')) {
			*val = (*val << 4) | (*tmp - 'A' + 10);
		}
		else {
			break;
		}

		tmp += 1;
		cnt += 1;
	}

	if (cnt > 0) {
		*str = tmp;
		return (1);
	}

	return (0);
}

int match_uint32 (const char **str, unsigned long *val)
{
	unsigned      base, dig;
	int           sign;
	const char    *tmp;

	match_space (str);

	tmp = *str;

	switch (*tmp) {
		case '-':
			tmp += 1;
			sign = 1;
			break;

		case '+':
			tmp += 1;
			sign = 0;
			break;

		default:
			sign = 0;
			break;
	}

	base = 10;

	if (*tmp == '0') {
		switch (tmp[1]) {
			case 'x':
			case 'X':
				tmp += 2;
				base = 16;
				break;

			case 'b':
			case 'B':
				tmp += 2;
				base = 2;
				break;

			case 'o':
			case 'O':
				tmp += 2;
				base = 2;
				break;

			case 'd':
			case 'D':
				tmp += 2;
				base = 2;
				break;

			default:
				base = 10;
				break;
		}
	}

	*val = 0;

	while (1) {
		if ((*tmp >= '0') && (*tmp <= '9')) {
			dig = (unsigned) (*tmp - '0');
		}
		else if ((*tmp >= 'a') && (*tmp <= 'f')) {
			dig = (unsigned) (*tmp - 'a' + 10);
		}
		else if ((*tmp >= 'A') && (*tmp <= 'F')) {
			dig = (unsigned) (*tmp - 'A' + 10);
		}
		else {
			break;
		}

		if (dig >= base) {
			return (0);
		}

		*val = base * *val + dig;

		tmp += 1;
	}

	if (sign) {
		*val = ~*val + 1;
	}

	*val &= 0xffffffff;

	*str = tmp;

	return (1);
}
