#!/bin/bash

case "$1" in
"--help")
	echo "asdec: decode AppleSingle files"
	echo
	echo "usage: asdec [options] [file.as...]"
	echo "  -d, --delete  Delete source files after decoding [no]"
	echo "  -r, --dump    Store resources as resource dump [no]"
	exit 0
	;;

"--version")
	echo "asdec (mac-utils) version %MACUTILS_VERSION_STR"
	echo ""
	echo "Copyright (C) 2009 Hampa Hug <hampa@hampa.ch>"
	exit 0
	;;
esac


# as_decode filename
as_decode ()
{
	local name=$1
	local bas ext

	case "$name" in
	*.*)
		bas=${name%.*}
		ext=${name##*.}
		;;
	*)
		echo "$0: bad file name ($name)" >&2
		return 1
		;;
	esac

	if test ! -f "$bas.$ext" ; then
		echo "$0: no such file ($bas.$ext)" >&2
		return 1
	fi

	if test ! -d ".mac" ; then
		mkdir -p ".mac"
	fi

	if ! asng -d -b "$bas.$ext" -f "$bas" -r ".mac/$bas.rsrc" -n ".mac/$bas.info" ; then
		echo "$0: error decoding $name" >&2
		return 1
	fi

	touch "--reference=$bas.$ext" ".mac/$bas.rsrc" > /dev/null 2>&1
	touch "--reference=$bas.$ext" ".mac/$bas.info" > /dev/null 2>&1

	if test "x$par_dump" = "x1" ; then
		macrsc -d -i ".mac/$bas.rsrc" -o ".mac/$bas.r"
		touch "--reference=.mac/$bas.rsrc" ".mac/$bas.r" > /dev/null 2>&1
		rm -f ".mac/$bas.rsrc"
	fi

	if test "x$par_del" = "x1" ; then
		rm -f "$bas.$ext"
	fi
}


par_del=0
par_dump=0

while test $# -gt 0 ; do
	case "$1" in
	"-d" | "--delete")
		par_del=1
		;;
	"-r" | "--dump")
		par_dump=1
		;;
	-*)
		echo "$0: unknown argument ($1)" >&2
		exit 1
		;;
	*)
		if test -f "$1" ; then
			as_decode "$1"
		fi
		;;
	esac

	shift 1
done
