#!/bin/bash

case "$1" in
"--help")
	echo "macenc: encode macintosh files"
	echo
	echo "usage: macenc [options] [file...]"
	echo "  -d, --delete  Delete source files after encoding [no]"
	echo "  -s, --sort    Sort the resources [no]"
	exit 0
	;;

"--version")
	echo "macenc (mac-utils) version %MACUTILS_VERSION_STR"
	echo ""
	echo "Copyright (C) 2011-2019 Hampa Hug <hampa@hampa.ch>"
	exit 0
	;;
esac


# mac_get_type file type creator
mac_get_type ()
{
	local file=$1
	local src
	local ext t c rest
	local tab

	for src in "$HOME/AppleVolumes" "$HOME/.AppleVolumes" "$HOME/applevolumes" "$HOME/.applevolumes" "/etc/netatalk/AppleVolumes.system" ; do
		if test -r "$src" ; then
			break;
		fi
	done

	if test ! -r "$src" ; then
		eval "$2=\"????\""
		eval "$3=\"????\""
		return 0
	fi

	tab=$(printf "\\t")

	while IFS=$tab read ext t c rest ; do
		case "$ext" in
		.*)
			case "$file" in
			*$ext)
				t=${t#\"}
				t=${t%\"}
				eval "$2=\"$t\""
				c=${c#\"}
				c=${c%\"}
				eval "$3=\"$c\""
				return 0
				;;
			esac
			;;
		esac
	done < "$src"

	eval "$2=\"????\""
	eval "$3=\"????\""
	return 0
}

# mac_encode file
mac_encode ()
{
	local bas=$1
	local data rsrc info
	local delrsc
	local type creator
	local opt

	case "$bas" in
	*.mac)
		echo "$0: not encoding a mar file, skipping $bas" >&2
		return 1
		;;
	esac

	if test -f "$bas.mac" ; then
		echo "$0: $bas.mac exists, skipping $bas" >&2
		return 1
	fi

	if test ! -d ".mac" ; then
		mkdir -p ".mac"
	fi

	delrsc=0

	if test -f ".mac/$bas.info" ; then
		info=".mac/$bas.info"
	elif test -f ".mac/$bas.inf" ; then
		info=".mac/$bas.inf"
	else
		info=""
	fi

	if test -f "$bas" ; then
		data=$bas
	elif test -f ".mac/$bas.data" ; then
		data=".mac/$bas.data"
	else
		touch "$bas"
		data=$bas
	fi

	if test -f ".mac/$bas.rsrc" ; then
		rsrc=".mac/$bas.rsrc"
	elif test -f ".mac/$bas.rsc" ; then
		rsrc=".mac/$bas.rsc"
	elif test -f ".mac/$bas.r" ; then
		opt=""
		if test "x$par_sort" = "x1" ; then
			opt="$opt -s"
		fi
		macrsc -e $opt -i ".mac/$bas.r" -o ".mac/$bas.rsrc"
		rsrc=".mac/$bas.rsrc"
		delrsc=1
	elif test -f "$bas.rsrc" ; then
		rsrc="$bas.rsrc"
	else
		touch ".mac/$bas.rsrc"
		rsrc=".mac/$bas.rsrc"
	fi

	if test -f "$info" ; then
		if ! mar -e -f "$bas.mac" -d "$data" -r "$rsrc" -i "$info" ; then
			echo "$0: error encoding $bas.mac" >&2
			return 1
		fi
	else
		mac_get_type "$bas" type creator

		if ! mar -e -f "$bas.mac" -d "$data" -r "$rsrc" -s type "$type" -s creator "$creator" -m "$bas" ; then
			echo "$0: error encoding $bas.mac" >&2
			return 1
		fi
	fi

	if test "x$delrsc" = "x1" ; then
		rm -f -- "$rsrc"
	fi

	if test "x$par_del" = "x1" ; then
		rm -f -- "$data" "$rsrc" "$info"
	fi
}


par_del=0
par_sort=0

r=0

while test $# -gt 0 ; do
	case "$1" in
	"-d" | "--delete")
		par_del=1
		;;
	"-s" | "--sort")
		par_sort=1
		;;
	-*)
		echo "$0: unknown argument ($1)" >&2
		exit 1
		;;
	*)
		if test ! -d "$1" ; then
			if ! mac_encode "$1" ; then
				r=1
			fi
		fi
		;;
	esac

	shift 1
done

for f in .mac/* ; do
	if test -f "$f" ; then
		exit $r
	fi
done

rm -rf ".mac"

exit $r
