#!/bin/bash

case "$1" in
"--help")
	echo "usage: marclean [options] [file...]"
	echo "  -s, --sort  Sort resources by type and id [no]"
	exit 0
	;;

"--version")
	echo "marclean (mac-utils) version %MACUTILS_VERSION_STR"
	echo ""
	echo "Copyright (C) 2011 Hampa Hug <hampa@hampa.ch>"
	exit 0
	;;
esac


mar_clean ()
{
	local mar=$1
	local bas
	local dat
	local rsc
	local inf
	local tmp
	local rscopt

	bas=${mar%.mac}
	bas=${bas##*/}

	if test ! -r "$mar" -o ! -d "$par_tmp" ; then
		return 1
	fi

	dat="$par_tmp/$bas.dat"
	rsc="$par_tmp/$bas.rsc"
	tmp="$par_tmp/$bas.tmp"
	inf="$par_tmp/$bas.inf"

	if ! mar -d -f "$mar" -d "$dat" -r "$rsc" -i "$inf" ; then
		rm -f "$dat" "$rsc" "$tmp" "$inf"
		return 1
	fi

	if test -s "$rsc" ; then
		if test "x$par_sort" = "x1" ; then
			rscopt="-s"
		else
			rscopt=""
		fi

		if ! macrsc -w $rscopt -i "$rsc" -o "$tmp" ; then
			rm -f "$dat" "$rsc" "$tmp" "$inf"
			return 1
		fi

		mar -e -f "$mar" -d "$dat" -r "$tmp" -i "$inf"

		mar -f -f "$mar"
	fi

	rm -f "$dat" "$rsc" "$tmp" "$inf"
}

par_tmp="/tmp/macrsc-$USER-$$"

par_sort=0

mkdir -p "$par_tmp"
chmod 700 "$par_tmp"

while test $# -gt 0 ; do
	case "$1" in
	"-s" | "--sort")
		par_sort=1
		;;
	-*)
		echo "$0: unknown option ($1)" >&2
		exit 1
		;;
	*)
		if ! mar_clean "$1" ; then
			echo "$0: $1 failed" >&2
		fi
		;;
	esac

	shift 1
done

rm -r -f "$par_tmp"
